#include "predef.h" 
#include <stdio.h>
#include <ctype.h>
#include <startnet.h>
#include <autoupdate.h>
#include <dhcpclient.h>
#include <..\mod5282\system\sim5282.h>
#include <cfinter.h>


extern "C" { 
void UserMain(void * pd);
extern const unsigned long CPU_CLOCK;
void SetIntc( int intc, long func, int vector, int level, int prio );
}

const char * AppName="bare_serial";



volatile uartstruct *BareUart;




/* Receiveing circular buffer stuff*/
OS_SEM SemDataRx;
char RxBuffer[1000];
volatile int char_put_index;
volatile int char_get_index;


/* Tranmitting buffer stuff.... */
const char *TxBuffer;
int TxLen;
OS_SEM SemDataTx;




INTERRUPT( BARE_ISR, 0x2400 )
{
while (BareUart->usr & 1)
	{//Process incomming chars */
	 
	 char c = ( BareUart->udb ); 
 

	 /* For this example we will store all the chars up to a \n in a circular buffer */
	 if (c!='\n')
	 {
		 
	 RxBuffer[char_put_index++]=c;
	 if (char_put_index==1000) char_put_index=0;
	 
	 if (char_put_index==char_get_index)
		{/* If we got here we overflowed our circular buffer and should handle the error */

		 char_get_index++;
		 if (char_get_index==1000) char_get_index=0;
		}
	 }
	 else
		 OSSemPost( &SemDataRx ); 
	}
if (BareUart->ir & 1==0) return; /* Don't do TX if we shouldn't be */
while (BareUart->usr & 4)
	{
	 if (TxLen)
		{
		 TxLen--;
		 BareUart->udb=*(TxBuffer++);
	    }
   		else
		{//TxLen got to zero so we need to turn off the ISR stuff
		 BareUart->ir=2; /* Only do RX interrupts */
		 BareUart->ucr = 0x08; /* TX disable */
		 OSSemPost(&SemDataTx);
		}
	}


}




void OpenBareSerial(int port, unsigned long baud)
{

/* Initialize the Semaphores */
OSSemInit(&SemDataRx,0);
OSSemInit(&SemDataTx,0); 

	
BareUart = &sim.uarts[port];

unsigned char ub1 = 0;
unsigned char ub2 = 0x41;

/* Calculate baud rate */
DWORD hdiv = ( ( ( CPU_CLOCK / 16 ) / baud ) + 1 ) >> 1;
ub1 = ( hdiv >> 8 ) & 0xFF;
ub2 = ( hdiv & 0xFF );

BareUart->ucr =  0x2a; /*Reset Rx Transmit disable Rx disable */
BareUart->ucr =  0x3a;    /*Reset Tx Transmit disable Rx disable */
BareUart->ucr =  0x1a;/*Transmit disable Rx disable reset mode reg */

BareUart->ucr =  0x10;
BareUart->umr =  0x13; /* umr1 8 data bits no parity */
BareUart->umr =  0x27;  /* umr2 1 stop*/ 

BareUart->usr =  0xDD; /* Use the internal timer for Baud Rate Generation */
BareUart->iacr = 0x00; 
BareUart->ir =   0x02;  /*Enable Rx  interrupts */

BareUart->dur = ub1; /* Actualy set the baud rate we calculated */
BareUart->dlr = ub2;


switch (port) /* Set up the hardware ports */
{
case 0:
	SetIntc( 0, ( unsigned long ) & BARE_ISR, 13, 3, 1 );    
    sim.gpio.puapar |= 0x03; /* 0000 0011 UM 26.3.2.15*/
    break;

case 1:
	SetIntc( 0, ( unsigned long ) & BARE_ISR, 14, 3, 2 );
	sim.gpio.puapar |= 0x0C; /*0000 1100 UM 26.3.2.15*/
	break;
case 2:
	{
	 WORD paspar = sim.gpio.paspar;
	 if ( ( paspar & 3 ) == 2 )
	 {
		 paspar &= 0xFFFC;
	 }
	 if ( ( paspar & 6 ) == 4 )
	 {
		 paspar &= 0xFFF3;
	 }
	 /* Third serial on CAN pins */
	 paspar &= 0xF0F; /* 0000 0000 1010 0000  UM 26.3.2.10*/ 
	 paspar |= 0xA0; /* 0000 0000 1010 0000  UM 26.3.2.10*/ 
     sim.gpio.paspar = paspar;
	 SetIntc( 0, ( unsigned long ) & BARE_ISR, 15, 3, 3 );


    }
	break;
}//End of switch


BareUart->ucr = 0x05;/*Enable Tx and Rx */ 
}


/* This write will block until all the data is in the uart */
void IsrDrivenWrite(int port, char * buffer, int len)
{
TxBuffer=buffer;
TxLen=len;

	
/* The TX ISR is turned off in the normal state.*/
sim.uarts[port].ir = 0x03; /* Enable the TX ISR */
sim.uarts[port].ucr = 4; /* Tx enable */ 
	
/* Wait until the tranmitter is done */
OSSemPend(&SemDataTx,0);
}






void UserMain(void * pd)
{
   InitializeStack();
   if (EthernetIP == 0) GetDHCPAddress(); 
   OSChangePrio(MAIN_PRIO);
   EnableAutoUpdate();

   OpenBareSerial(1,115200);

   iprintf("Application started\n");
   while (1)
   {  int put_index,get_index;

	   OSSemPend(&SemDataRx,0);
	   /* We don't want to be interrupted while we get the indexes */
	   USER_ENTER_CRITICAL();
	   put_index=char_put_index; 
	   get_index=char_get_index; 
	   USER_EXIT_CRITICAL();
	   

	   /* Now to print out our circular buffer */
	   iprintf("We got a buffer[");
	   {
	   while (get_index!=put_index)
		{
		 iprintf("%c",RxBuffer[get_index++]);
		 if (get_index==1000) get_index=0;
		}
	   char_get_index=get_index; 
	   iprintf("]\r\n");
	   IsrDrivenWrite(1,"Done\r\n",6);
	   }
  }
}
